# -*- coding: utf-8 -*-
from base64 import b64encode
from collective.restapi.pam.testing import COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING  # noqa
from DateTime import DateTime
from datetime import datetime
from datetime import timedelta
from freezegun import freeze_time
from plone import api
from plone.app.discussion.interfaces import IConversation
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.app.discussion.interfaces import IReplies
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.testing import applyProfile
from plone.app.testing import popGlobalRegistry
from plone.app.testing import pushGlobalRegistry
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.locking.interfaces import ITTWLockable
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from plone.registry.interfaces import IRegistry
from plone.restapi.testing import register_static_uuid_utility
from plone.restapi.testing import RelativeSession
from plone.testing.z2 import Browser
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.site.hooks import getSite

import collections
import json
import os
import pkg_resources
import re
import transaction
import unittest


try:
    # p.a.multilingual < 1.2
    pkg_resources.get_distribution('plone.multilingual')
    from plone.multilingual.interfaces import ITranslationManager
    from plone.multilingual.interfaces import ILanguage

except pkg_resources.DistributionNotFound:
    # p.a.multilingual >1.1, < 2
    from plone.app.multilingual.interfaces import ITranslationManager
    from plone.app.multilingual.interfaces import ILanguage

REQUEST_HEADER_KEYS = [
    'accept',
    'authorization',
    'lock-token',
]

RESPONSE_HEADER_KEYS = [
    'content-type',
    'allow',
    'location',
]

base_path = os.path.join(
    os.path.dirname(__file__),
    '..',
    '..',
    '..',
    '..',
    '..',
    'docs/source/_json'
)


def pretty_json(data):
    return json.dumps(data, sort_keys=True, indent=4, separators=(',', ': '))


def save_request_and_response_for_docs(name, response):
    with open('{}/{}'.format(base_path, '%s.req' % name), 'w') as req:
        req.write('{} {} HTTP/1.1\n'.format(
            response.request.method,
            response.request.path_url
        ))
        ordered_request_headers = collections.OrderedDict(
            sorted(response.request.headers.items())
        )
        for key, value in ordered_request_headers.items():
            if key.lower() in REQUEST_HEADER_KEYS:
                req.write('{}: {}\n'.format(key.title(), value))
        if response.request.body:
            # If request has a body, make sure to set Content-Type header
            if 'content-type' not in REQUEST_HEADER_KEYS:
                content_type = response.request.headers['Content-Type']
                req.write('Content-Type: %s\n' % content_type)

            req.write('\n')

            # Pretty print JSON request body
            if content_type == 'application/json':
                json_body = json.loads(response.request.body)
                body = pretty_json(json_body)
                # Make sure Content-Length gets updated, just in case we
                # ever decide to dump that header
                response.request.prepare_body(data=body, files=None)

            req.write(response.request.body)

    with open('{}/{}'.format(base_path, '%s.resp' % name), 'w') as resp:
        status = response.status_code
        reason = response.reason
        resp.write('HTTP/1.1 {} {}\n'.format(status, reason))
        for key, value in response.headers.items():
            if key.lower() in RESPONSE_HEADER_KEYS:
                resp.write('{}: {}\n'.format(key.title(), value))
        resp.write('\n')
        resp.write(response.content)


class TestDocumentation(unittest.TestCase):

    layer = COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()

        # Register custom UUID generator to produce stable UUIDs during tests
        pushGlobalRegistry(getSite())
        register_static_uuid_utility(prefix='SomeUUID')


        self.time_freezer = freeze_time("2016-10-21 19:00:00")
        self.frozen_time = self.time_freezer.start()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({'Accept': 'application/json'})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        language_tool = api.portal.get_tool('portal_languages')
        language_tool.addSupportedLanguage('en')
        language_tool.addSupportedLanguage('es')
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal['en'].invokeFactory(
            'Document',
            id='test-document',
            title='Test document'
        )
        self.en_content = self.portal['en'].get(en_id)

        es_id = self.portal['es'].invokeFactory(
            'Document',
            id='test-document',
            title='Test document'
        )
        self.es_content = self.portal['es'].get(es_id)

        import transaction
        transaction.commit()
        self.browser = Browser(self.app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )

    def tearDown(self):
        self.time_freezer.stop()
        popGlobalRegistry(getSite())

    def test_documentation_translations_post(self):
        response = self.api_session.post(
            '{}/@translations'.format(self.en_content.absolute_url()),
            json={
                'id': self.es_content.absolute_url()
            }
        )
        save_request_and_response_for_docs('translations_post', response)

    def test_documentation_translations_get(self):
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)
        transaction.commit()
        response = self.api_session.get(
            '{}/@translations'.format(self.en_content.absolute_url()))

        save_request_and_response_for_docs('translations_get', response)

    def test_documentation_translations_delete(self):
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)
        transaction.commit()
        response = self.api_session.delete(
            '{}/@translations'.format(self.en_content.absolute_url()),
            json={
                "language": "es"
            })
        save_request_and_response_for_docs('translations_delete', response)

    def test_translations_is_expandable(self):
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)
        transaction.commit()
        response = self.api_session.get(self.en_content.absolute_url())

        save_request_and_response_for_docs('translations_is_expandable', response)

    def test_expand_translations(self):
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)
        transaction.commit()
        response = self.api_session.get(
            self.en_content.absolute_url() + '?expand=translations')

        save_request_and_response_for_docs('expand_translations', response)



# -*- coding: utf-8 -*-
from OFS.Folder import Folder
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.testing import z2
from plone.uuid.interfaces import IUUIDGenerator
from Testing import ZopeTestCase as ztc
from zope.component import getGlobalSiteManager
from zope.interface import implements

import collective.restapi.pam
import transaction

class CollectiveRestapiPamLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.contenttypes
        import plone.multilingual
        import plone.app.multilingual
        import plone.multilingualbehavior
        import archetypes.multilingual
        import plone.restapi
        self.loadZCML(package=plone.app.contenttypes)
        self.loadZCML(package=plone.multilingual)
        self.loadZCML(package=plone.app.multilingual)
        self.loadZCML(package=archetypes.multilingual)
        self.loadZCML(package=plone.multilingualbehavior)
        self.loadZCML(package=plone.restapi)

        self.loadZCML(package=collective.restapi.pam)

        z2.installProduct(app, 'Products.DateRecurringIndex')
        z2.installProduct(app, 'plone.app.dexterity')
        z2.installProduct(app, 'plone.app.contenttypes')
        z2.installProduct(app, 'plone.multilingual')
        z2.installProduct(app, 'plone.app.multilingual')
        z2.installProduct(app, 'archetypes.multilingual')
        z2.installProduct(app, 'plone.multilingualbehavior')
        z2.installProduct(app, 'plone.restapi')

        # Support sessionstorage in tests
        app.REQUEST['SESSION'] = self.Session()
        if not hasattr(app, 'temp_folder'):
            tf = Folder('temp_folder')
            app._setObject('temp_folder', tf)
            transaction.commit()

        ztc.utils.setupCoreSessions(app)


    def setUpPloneSite(self, portal):

        portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ['Manager'], [])

        login(portal, SITE_OWNER_NAME)

        if portal.portal_setup.profileExists('plone.multilingual:default'):
            applyProfile(portal, 'plone.multilingual:default')
        if portal.portal_setup.profileExists('archetypes.multilingual:default'):  # noqa
            applyProfile(portal, 'archetypes.multilingual:default')
        if portal.portal_setup.profileExists('plone.multilingualbehavior:default'):  # noqa
            applyProfile(portal, 'plone.multilingualbehavior:default')

        applyProfile(portal, 'plone.app.dexterity:default')
        applyProfile(portal, 'plone.app.contenttypes:default')
        applyProfile(portal, 'plone.app.multilingual:default')
        applyProfile(portal, 'plone.restapi:default')

        portal.portal_languages.addSupportedLanguage('en')
        portal.portal_languages.addSupportedLanguage('es')
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")


COLLECTIVE_RESTAPI_PAM_FIXTURE = CollectiveRestapiPamLayer()


COLLECTIVE_RESTAPI_PAM_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_RESTAPI_PAM_FIXTURE,),
    name='CollectiveRestapiPamLayer:IntegrationTesting'
)


COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_RESTAPI_PAM_FIXTURE, z2.ZSERVER_FIXTURE),
    name='CollectiveRestapiPamLayer:FunctionalTesting'
)



# -*- coding: utf-8 -*-
from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from Products.CMFCore.utils import getToolByName
from zope.component import adapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import implementer

import plone.protect.interfaces
import pkg_resources

try:
    # p.a.multilingual < 1.2
    pkg_resources.get_distribution('plone.multilingual')
    from plone.multilingual.interfaces import ILanguage
    from plone.multilingual.interfaces import ITranslatable
    from plone.multilingual.interfaces import ITranslationManager

except pkg_resources.DistributionNotFound:
    # p.a.multilingual >1.1, < 2
    from plone.app.multilingual.interfaces import ILanguage
    from plone.app.multilingual.interfaces import ITranslatable
    from plone.app.multilingual.interfaces import ITranslationManager


@implementer(IExpandableElement)
@adapter(ITranslatable, Interface)
class Translations(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {
            'translations': {
                '@id': '{}/@translations'.format(self.context.absolute_url()),
            },
        }
        if not expand:
            return result

        translations = []
        manager = ITranslationManager(self.context)
        for language, translation in manager.get_translations().items():
            if language != ILanguage(self.context).get_language():
                translations.append({
                    '@id': translation.absolute_url(),
                    'language': language,
                })

        result['translations']['items'] = translations
        return result


class TranslationInfo(Service):
    """ Get translation information
    """

    def reply(self):
        translations = Translations(self.context, self.request)
        return translations(expand=True)['translations']


class LinkTranslations(Service):
    """ Link two content objects as translations of each other
    """

    def reply(self):
        # Disable CSRF protection
        if 'IDisableCSRFProtection' in dir(plone.protect.interfaces):
            alsoProvides(self.request,
                         plone.protect.interfaces.IDisableCSRFProtection)

        data = json_body(self.request)
        id_ = data.get('id', None)
        if id_ is None:
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='BadRequest',
                message='Missing content id to link to'))

        target = self._traverse(id_)
        if target is None:
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='BadRequest',
                message='Content does not exist'))

        target_language = ILanguage(target).get_language()
        manager = ITranslationManager(self.context)
        current_translation = manager.get_translation(target_language)
        if current_translation is not None:
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='BadRequest',
                message='Already translated into language {}'.format(
                    target_language)))

        manager.register_translation(target_language, target)
        self.request.response.setStatus(201)
        self.request.response.setHeader(
            'Location', self.context.absolute_url())
        return {}

    def _traverse(self, url):
        purl = getToolByName(self.context, 'portal_url')
        portal = purl.getPortalObject()
        portal_url = portal.absolute_url()
        if url.startswith(portal_url):
            content_path = url[len(portal_url)+1:]
            content_path = content_path.split('/')
            content_item = portal.restrictedTraverse(content_path)
            return content_item

        return None


class UnlinkTranslations(Service):
    """ Unlink the translations for a content object
    """

    def reply(self):
        # Disable CSRF protection
        if 'IDisableCSRFProtection' in dir(plone.protect.interfaces):
            alsoProvides(self.request,
                         plone.protect.interfaces.IDisableCSRFProtection)

        data = json_body(self.request)
        manager = ITranslationManager(self.context)
        language = data.get('language', None)
        if language is None:
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='BadRequest',
                message='You need to provide the language to unlink'))

        if language not in manager.get_translations().keys():
            self.request.response.setStatus(400)
            return dict(error=dict(
                type='BadRequest',
                message='This object is not translated into {}'.format(
                    language)))

        manager.remove_translation(language)
        self.request.response.setStatus(204)
        return {}



from collective.restapi.pam.testing import COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.serializer.expansion import expandable_elements
from plone.restapi.testing import RelativeSession
from zope.component import getMultiAdapter
from zope.component import provideAdapter
from zope.interface import Interface
from zope.interface import providedBy
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserRequest

import unittest
import transaction
import pkg_resources

try:
    # p.a.multilingual < 1.2
    pkg_resources.get_distribution('plone.multilingual')
    from plone.multilingual.interfaces import ITranslationManager

except pkg_resources.DistributionNotFound:
    # p.a.multilingual >1.1, < 2
    from plone.app.multilingual.interfaces import ITranslationManager


class TestExpansionFunctional(unittest.TestCase):

    layer = COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.portal_url = self.portal.absolute_url()
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)

        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({'Accept': 'application/json'})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        transaction.commit()

    def test_translations_is_expandable(self):
        response = self.api_session.get('/en/test-document')

        self.assertEqual(response.status_code, 200)
        self.assertIn(
            'translations',
            response.json().get('@components').keys()
        )

    def test_translations_expanded(self):
        response = self.api_session.get(
            '/en/test-document',
            params={
                "expand": "translations"
            }
        )

        self.assertEqual(response.status_code, 200)
        translation_dict = {
            '@id': self.es_content.absolute_url(),
            'language': 'es'
        }
        self.assertIn(
            translation_dict,
            response.json()['@components']['translations']['items']
        )



# -*- coding: utf-8 -*-
from base64 import b64encode
from collective.restapi.pam.testing import COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING  # noqa
from collective.restapi.pam.testing import COLLECTIVE_RESTAPI_PAM_INTEGRATION_TESTING  # noqa
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.testing import login
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from unittest import TestCase
from zope.component import getMultiAdapter
from zope.event import notify
from ZPublisher.pubevents import PubStart

import pkg_resources
import requests
import transaction

try:
    # p.a.multilingual < 1.2
    pkg_resources.get_distribution('plone.multilingual')
    from plone.multilingual.interfaces import ITranslationManager
    from plone.multilingual.interfaces import ILanguage

except pkg_resources.DistributionNotFound:
    # p.a.multilingual >1.1, < 2
    from plone.app.multilingual.interfaces import ITranslationManager
    from plone.app.multilingual.interfaces import ILanguage


class TestTranslationInfo(TestCase):

    layer = COLLECTIVE_RESTAPI_PAM_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)
        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)

    def test_translation_info_includes_translations(self):
        tinfo = getMultiAdapter(
            (self.en_content, self.request),
            name=u'GET_application_json_@translations')

        info = tinfo.reply()
        self.assertIn('items', info)
        self.assertEqual(1, len(info['items']))

    def test_correct_translation_information(self):
        tinfo = getMultiAdapter(
            (self.en_content, self.request),
            name=u'GET_application_json_@translations')

        info = tinfo.reply()
        tinfo_es = info['items'][0]
        self.assertEqual(
            self.es_content.absolute_url(),
            tinfo_es['@id'])
        self.assertEqual(self.es_content.Language(), tinfo_es['language'])


class TestLinkContents(TestCase):
    layer = COLLECTIVE_RESTAPI_PAM_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)
        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)

        untranslated_en_id = self.portal.en.invokeFactory(
            id='untranslated-test-document',
            type_name='Document',
            title='Test document'
        )
        self.untranslated_en_content = self.portal.en.get(untranslated_en_id)
        untranslated_es_id = self.portal.es.invokeFactory(
            id='untranslated-test-document',
            type_name='Document',
            title='Test document'
        )
        self.untranslated_es_content = self.portal.es.get(untranslated_es_id)

    def traverse(self, path='/plone', accept='application/json',
                 method='POST'):
        request = self.layer['request']
        request.environ['PATH_INFO'] = path
        request.environ['PATH_TRANSLATED'] = path
        request.environ['HTTP_ACCEPT'] = accept
        request.environ['REQUEST_METHOD'] = method
        request._auth = 'Basic %s' % b64encode(
            '%s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD))
        notify(PubStart(request))
        return request.traverse(path)

    def test_translation_link_without_id_gives_400(self):
        service = self.traverse('/plone/en/test-document/@translations')
        res = service.reply()
        self.assertEqual(400, self.request.response.getStatus())
        self.assertEqual(
            'Missing content id to link to', res['error']['message'])

    def test_translation_link_with_invalid_id_gives_400(self):
        self.request['BODY'] = '{"id": "http://server.com/unexisting-id"}'
        service = self.traverse('/plone/en/test-document/@translations')
        res = service.reply()
        self.assertEqual(400, self.request.response.getStatus())
        self.assertEqual(
            'Content does not exist', res['error']['message'])

    def test_translation_link_already_translated(self):
        self.request['BODY'] = '{"id": "http://nohost/plone/es/test-document"}'
        service = self.traverse('/plone/en/test-document/@translations')
        res = service.reply()
        self.assertEqual(400, self.request.response.getStatus())
        self.assertEqual(
            'Already translated into language es', res['error']['message'])

    def test_translation_linking_succeeds(self):
        self.request['BODY'] = '{"id": "http://nohost/plone/es/untranslated-test-document"}' # noqa
        service = self.traverse(
            '/plone/en/untranslated-test-document/@translations')
        service.reply()
        self.assertEqual(201, self.request.response.getStatus())


class TestUnLinkContents(TestCase):
    layer = COLLECTIVE_RESTAPI_PAM_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)
        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)

        untranslated_en_id = self.portal.en.invokeFactory(
            id='untranslated-test-document',
            type_name='Document',
            title='Test document'
        )
        self.untranslated_en_content = self.portal.en.get(untranslated_en_id)
        untranslated_es_id = self.portal.es.invokeFactory(
            id='untranslated-test-document',
            type_name='Document',
            title='Test document'
        )
        self.untranslated_es_content = self.portal.es.get(untranslated_es_id)

    def traverse(self, path='/plone', accept='application/json',
                 method='POST'):
        request = self.layer['request']
        request.environ['PATH_INFO'] = path
        request.environ['PATH_TRANSLATED'] = path
        request.environ['HTTP_ACCEPT'] = accept
        request.environ['REQUEST_METHOD'] = method
        request._auth = 'Basic %s' % b64encode(
            '%s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD))
        notify(PubStart(request))
        return request.traverse(path)


    def test_translation_unlink_invalid_language_returns_400(self):
        self.request['BODY'] = '{"language": "fr"}'
        service = self.traverse(
            '/plone/en/test-document/@translations', method='DELETE')
        res = service.reply()
        self.assertEqual(400, self.request.response.getStatus())
        self.assertEqual(
            'This object is not translated into fr', res['error']['message'])

    def test_translation_unlink_without_language_returns_400(self):
        service = self.traverse(
            '/plone/en/test-document/@translations', method='DELETE')
        res = service.reply()
        self.assertEqual(400, self.request.response.getStatus())
        self.assertEqual(
            'You need to provide the language to unlink',
            res['error']['message']
        )

    def test_translation_unlinking_succeeds(self):
        self.request['BODY'] = '{"language": "es"}'
        service = self.traverse(
            '/plone/en/test-document/@translations', method='DELETE')
        res = service.reply()
        self.assertEqual(204, self.request.response.getStatus())


class TestLinkContentsFunctional(TestCase):
    layer = COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)
        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        transaction.commit()

    def test_translation_linking_succeeds(self):
        response = requests.post(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
                'id': self.es_content.absolute_url(),
            },
        )
        self.assertEqual(201, response.status_code)
        transaction.begin()
        manager = ITranslationManager(self.en_content)
        for language, translation in manager.get_translations():
            if language == ILanguage(self.es_content).get_language():
                self.assertEqual(translation, self.es_content)

    def test_calling_endpoint_without_id_gives_400(self):
        response = requests.post(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
            },
        )
        self.assertEqual(400, response.status_code)

    def test_calling_endpoint_with_invalid_id_gives_400(self):
        response = requests.post(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
                'id': 'http://server.com/this-content-does-not-exist'
            },
        )
        self.assertEqual(400, response.status_code)


class TestUnLinkContentsFunctional(TestCase):
    layer = COLLECTIVE_RESTAPI_PAM_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        # self.portal.portal_languages.addSupportedLanguage('en')
        # self.portal.portal_languages.addSupportedLanguage('es')
        #  Setup the language root folders
        login(self.portal, SITE_OWNER_NAME)
        # Setup language root folders
        setupTool = SetupMultilingualSite()
        setupTool.setupSite(self.portal)

        en_id = self.portal.en.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.en_content = self.portal.en.get(en_id)
        es_id = self.portal.es.invokeFactory(
            id='test-document',
            type_name='Document',
            title='Test document'
        )
        self.es_content = self.portal.es.get(es_id)
        ITranslationManager(self.en_content).register_translation(
            'es', self.es_content)
        transaction.commit()

    def test_translation_unlinking_succeeds(self):
        response = requests.delete(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
                'language': 'es',
            },
        )
        self.assertEqual(204, response.status_code)
        transaction.begin()
        manager = ITranslationManager(self.en_content)
        for language, translation in manager.get_translations():
            if language == ILanguage(self.es_content).get_language():
                self.assertEqual(translation, self.es_content)

    def test_calling_endpoint_without_language_gives_400(self):
        response = requests.delete(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
            },
        )
        self.assertEqual(400, response.status_code)

    def test_calling_with_an_untranslated_content_gives_400(self):
        ITranslationManager(self.en_content).remove_translation("es")
        transaction.commit()
        response = requests.delete(
            '{}/@translations'.format(self.en_content.absolute_url()),
            headers={'Accept': 'application/json'},
            auth=(SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
            json={
                'language': 'es',
            },
        )
        self.assertEqual(400, response.status_code)



# -*- coding: utf-8 -*-
"""Installer for the collective.restapi.pam package."""

from setuptools import find_packages
from setuptools import setup


long_description = '\n\n'.join([
    open('README.rst').read(),
    open('CONTRIBUTORS.rst').read(),
    open('CHANGES.rst').read(),
])


setup(
    name='collective.restapi.pam',
    version='2.0.0',
    description="An add-on providing plone.restapi endpoint for translations handled using plone.app.multilingual",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords='Python Plone',
    author='Mikel Larreategi',
    author_email='mlarreategi@codesyntax.com',
    url='https://pypi.python.org/pypi/collective.restapi.pam',
    license='GPL version 2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective', 'collective.restapi'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'plone.multilingualbehavior',
        'archetypes.multilingual <2',
        'plone.restapi',
        'plone.app.dexterity',
        'plone.app.multilingual <=2.99',
        'Plone <= 4.99',
    ],
    extras_require={
        'test': [
            'collective.MockMailHost',
            'freezegun',
            'plone.api',
            'plone.app.testing [robot] >= 4.2.2',
            'plone.testing',
            'plone.app.contenttypes <1.2',
            'plone.app.robotframework[debug]',
            'requests',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
